@@ -1,6 +1,6 @@
 # rpart.model.frame.R: The code that accesses the rpart model's model.frame
 #
-# Includes processing the roundint argument and is.logical determination
+# Includes processing the roundint argument and islogical determination
 # for the variables.
 # We have to semi-gracefully handle the situation where the data used
 # to build the model is not longer available.
@@ -13,15 +13,16 @@ get.modelframe.info <- function(obj, roundint, trace, env, caller)
     trace <- as.numeric(check.numeric.scalar(trace, logical.ok=TRUE))
     response.name <- get.response.name(obj, trace)
     model.frame <- rpart.model.frame(obj, roundint, trace, env, caller)
-    is.logical <- NA
+    isbinary <- list(islogical=NA, is01=NA)
     if(!identical(model.frame, NA))
-        is.logical <- get.is.logical(obj, model.frame, trace)
-    is.roundint <- NA
+        isbinary <- isbinary(obj, model.frame, trace)
+    isroundint <- NA
     if(roundint && !identical(model.frame, NA))
-        is.roundint <- get.is.roundint(model.frame, trace, caller)
+        isroundint <- isroundint(model.frame, trace, caller)
     list(response.name = response.name,
-         is.logical    = is.logical,  # NA or a logical vec, elem for each split
-         is.roundint   = is.roundint) # NA or a logical vec, elem names are varnames
+         islogical     = isbinary$islogical, # NA or a logical vec, elem for each split
+         is01          = isbinary$is01,      # NA or a logical vec, elem for each split
+         isroundint    = isroundint)         # NA or a logical vec, elem names are varnames
 }
 get.response.name <- function(obj, trace) # return a string
 {
@@ -58,47 +59,54 @@ get.response.name <- function(obj, trace) # return a string
         response.name <- "Surv" # so rules not pushed far to the right with long response.name
     response.name
 }
+# islogical is true if is.logical is TRUE for the variable
+# is01      is true if the variable only has values 0 and 1
 # return a single NA, or a logical vec with an element for each split
-get.is.logical <- function(obj, model.frame, trace)
+isbinary <- function(obj, model.frame, trace)
 {
     rownames <- rownames(obj$splits)
-    if(length(rownames) == 0) # null model, no splits
-        return(NA)
-    is.logical <- repl(FALSE, length(rownames))
+    if(length(rownames) == 0)   # null model, no splits
+        return(list(islogical=NA, is01=NA))
+    islogical <- repl(FALSE, length(rownames))
+    is01      <- repl(FALSE, length(rownames))
     for(irow in 1:length(rownames)) {
         x <- model.frame[[rownames[irow]]]
-        if(!is.null(x))      # rowname is in model.frame?
-            is.logical[irow] <- is.logical(x)
-        else if(trace > 0) { # rowname not in model.frame
+        if(!is.null(x)) {       # rowname is in model.frame?
+            islogical[irow] <- is.logical(x)
+            u <- sort(unique(x))
+            is01[irow] <-
+                (length(u) == 1 && (u == 0 || u == 1)) ||
+                (length(u) == 2 && (u[1] == 0 && u[2] == 1))
+        } else if(trace > 0) {  # rowname not in model.frame
             # get here in e.g. model dfit in slowtests/rpart.report.R
             trace <- 0 # prevent further messages
             printf(
-"Variable '%s' in splits is not in the model.frame (so cannot determine is.logical for the variable)\n",
+"Variable '%s' in splits is not in the model.frame (so cannot determine roundint and is.binary for the variable)\n",
                    rownames[irow])
             printf("colnames(model.frame): %s\n", paste.collapse(colnames(model.frame)))
         }
     }
-    is.logical
+    list(islogical=islogical, is01=is01)
 }
 # return a single NA, or a logical vec, element names are varnames.
-get.is.roundint <- function(model.frame, trace, caller)
+isroundint <- function(model.frame, trace, caller)
 {
     varnames <- colnames(model.frame)
-    is.roundint <- repl(FALSE, length(varnames))
-    names(is.roundint) <- varnames
+    isroundint <- repl(FALSE, length(varnames))
+    names(isroundint) <- varnames
     for(varname in varnames) {
         x <- model.frame[[varname]] # will be NULL if varname not in model.frame
-        is.roundint[varname] <- (is.numeric(x) || is.logical(x)) && is.integral(x)
+        isroundint[varname] <- (is.numeric(x) || is.logical(x)) && is.integral(x)
     }
     if(trace >= 2) {
-        varnames <- names(is.roundint)[is.roundint]
+        varnames <- names(isroundint)[isroundint]
         if(length(varnames))
             cat("will apply roundint to the following variables in splits:",
                 varnames, "\n")
         else
             cat("will apply roundint to no variables\n")
     }
-    is.roundint # logical vector, element names are varnames
+    isroundint # logical vector, element names are varnames
 }
 # Return NA if can't get the model frame.
 # We issue warnings only if roundint is TRUE (this allows the user to silence the warnings).
@@ -152,17 +160,18 @@ rpart.model.frame <- function(obj, roundint, trace, env, caller)
     }
     data <- call[["data"]]
     if(!is.null(data)) { # null if data arg not used in original call to rpart
-        if(!inherits(data, "name")) {
-            msg("Bad 'data' field in model 'call'")
-            return(NA)
-        }
-        data <- try(eval(data, envir=env), silent=TRUE)
-        # will be a try error here if legitimate model but data no longer available
-        if(is.try.err(data) ||
-                length(dim(data)) != 2 ||
-                NROW(data) < 1 || NCOL(data) < 2) {
-            msg("Cannot retrieve the data used to build the model ",
-                "(so cannot determine roundint and is.logical for the variables)")
+        if(inherits(data, "name")) {
+            data <- try(eval(data, envir=env), silent=TRUE)
+            # will be a try error here if legitimate model but data no longer available
+            if(is.try.err(data) ||
+                    length(dim(data)) != 2 ||
+                    NROW(data) < 1 || NCOL(data) < 2) {
+                msg("Cannot retrieve the data used to build the model ",
+                    "(so cannot determine roundint and is.binary for the variables)")
+                return(NA)
+            }
+        } else if(!is.data.frame(data) && !is.matrix(data)) {
+            msg("Bad 'data' field in model 'call' (expected a data.frame or a matrix)")
             return(NA)
         }
     }
@@ -174,6 +183,12 @@ rpart.model.frame <- function(obj, roundint, trace, env, caller)
             attr(model.frame,"condition")[[1]], ")")
        return(NA)
     }
+    if(trace >= 3) {
+        old.width <- options(width=1e4)$width
+        on.exit(options(width=old.width))
+        printf("head(model.frame):\n")
+        print(head(model.frame))
+    }
     if(length(dim(model.frame)) != 2 ||
             NROW(model.frame) < 1 || NCOL(model.frame) < 2) {
         msg("Cannot retrieve the data used to build the model ",
@@ -186,42 +201,42 @@ rpart.model.frame <- function(obj, roundint, trace, env, caller)
     }
     model.frame
 }
-# This was supplanted by get.is.logical because I wanted to treat roundint
-# and is.logical more uniformly (using the model.frame not dataClasses)
+# This was supplanted by isbinary because I wanted to treat roundint
+# and islogical more uniformly (using the model.frame not dataClasses)
 #
-# get.is.logical.from.dataClasses <- function(obj, trace)
+# isbinary.from.dataClasses <- function(obj, trace)
 # {
 #     terms <- try(terms(obj), silent=TRUE)
 #     if(is.try.err(terms)) {
-#         trace1(trace, "%s (therefore cannot determine is.logical for the variables)\n",
+#         trace1(trace, "%s (therefore cannot determine islogical for the variables)\n",
 #                attr(terms,"condition")[[1]])
 #         return(NA)
 #     }
 #     dataClasses <- attr(terms, "dataClasses")
 #     if(!is.character(dataClasses)) {
 #         trace1(trace,
-# "Cannot get dataClasses (therefore cannot determine is.logical for the variables)\n")
+# "Cannot get dataClasses (therefore cannot determine islogical for the variables)\n")
 #         return(NA)
 #     }
 #     rownames <- rownames(obj$splits)
 #     if(length(rownames) == 0) # null model, no splits
 #         return(NA)
-#     is.logical <- repl(FALSE, length(rownames))
+#     islogical <- repl(FALSE, length(rownames))
 #     first.time <- TRUE
 #     for(irow in 1:length(rownames)) {
 #         classname <- dataClasses[rownames[irow]][1]
 #         if(!is.na(classname)) {
 #             if(classname == "logical")
-#                 is.logical[irow] <- TRUE
+#                 islogical[irow] <- TRUE
 #         } else if(trace > 0 && first.time) {
 #             first.time <- FALSE
 #             # get here in "dfit" in Section 4 in slowtests/rpart.report.R
 #             printf(
-# "Variable '%s' in splits is not in dataClasses (therefore cannot determine is.logical for the variable)\nterms$dataClasses:\n",
+# "Variable '%s' in splits is not in dataClasses (therefore cannot determine islogical for the variable)\nterms$dataClasses:\n",
 #                    rownames[irow])
 #             print(dataClasses)
 #             cat0("\n")
 #         }
 #     }
-#     is.logical
+#     islogical
 # }
